Index: Doc/Zsh/builtins.yo
===================================================================
RCS file: /cvsroot/zsh/zsh/Doc/Zsh/builtins.yo,v
retrieving revision 1.78
diff -u -p -r1.78 builtins.yo
--- Doc/Zsh/builtins.yo	1 Jun 2005 10:45:41 -0000	1.78
+++ Doc/Zsh/builtins.yo	8 Aug 2005 14:44:38 -0000
@@ -1154,16 +1154,24 @@ and for processes run from the shell.
 findex(trap)
 cindex(signals, trapping)
 cindex(trapping signals)
-item(tt(trap) [ var(arg) [ var(sig) ... ] ])(
+item(tt(trap) [ var(arg) ] [ var(sig) ... ])(
 var(arg) is a series of commands (usually quoted to protect it from
 immediate evaluation by the shell) to be read and executed when the shell
-receives var(sig).  Each var(sig) can be given as a number,
-or as the name of a signal either with or without the string tt(SIG)
+receives any of the signals var(sig).  Each var(sig) can be given as a
+number, or as the name of a signal either with or without the string tt(SIG)
 in front.
-If var(arg) is `tt(-)', then all traps var(sig) are reset to their
+If var(arg) is `tt(-)', then the traps var(sig) are reset to their
 default values.  If var(arg) is the empty string, then this signal
 is ignored by the shell and by the commands it invokes.
 
+If var(sig) is omitted and var(arg) is `tt(-)', then all traps are reset
+to their defaults.  If var(arg) is omitted (i.e., the first argument is
+a valid signal number or name), the effect is the same as if var(arg)
+were `tt(-)'.
+
+The tt(trap) command with no arguments prints a list of commands
+associated with each signal.
+
 If var(sig) is tt(ZERR) then var(arg) will be executed
 after each command with a nonzero exit status.  tt(ERR) is an alias
 for tt(ZERR) on systems that have no tt(SIGERR) signal (this is the
@@ -1177,10 +1185,7 @@ If var(sig) is tt(0) or tt(EXIT)
 and the tt(trap) statement is not executed inside the body of a function,
 then the command var(arg) is executed when the shell terminates.
 
-tt(ZERR), tt(DEBUG) and tt(EXIT) traps are not executed inside other traps.
-
-The tt(trap) command with no arguments prints a list of commands
-associated with each signal.
+tt(ZERR), tt(DEBUG), and tt(EXIT) traps are not executed inside other traps.
 
 Note that traps defined with the tt(trap) builtin are slightly different
 from those defined as `tt(TRAP)var(NAL) () { ... }', as the latter have
Index: Src/builtin.c
===================================================================
RCS file: /cvsroot/zsh/zsh/Src/builtin.c,v
retrieving revision 1.143
diff -u -p -r1.143 builtin.c
--- Src/builtin.c	1 Aug 2005 22:20:36 -0000	1.143
+++ Src/builtin.c	8 Aug 2005 14:45:57 -0000
@@ -5021,9 +5021,15 @@ bin_trap(char *name, char **argv, UNUSED
 	    for (sig = 0; sig < VSIGCOUNT; sig++)
 		unsettrap(sig);
 	else
-	    while (*argv)
-		unsettrap(getsignum(*argv++));
-	return 0;
+	    for (; *argv; argv++) {
+		sig = getsignum(*argv);
+		if (sig == -1) {
+		    zwarnnam(name, "undefined signal: %s", *argv, 0);
+		    break;
+		}
+		unsettrap(sig);
+	    }
+	return *argv != NULL;
     }
 
     /* Sort out the command to execute on trap */
